Name:           mailfront-plugin-check-helo
Summary:        A mailfront plugin to provide an ability to reject, defer or add it's helo name checking results to message header
Version:        0.07
Release:        1%{?dist}
License:        GPL
Group:          Utilities/System
Source:         http://choon.net/opensource/mailfront-plugin-check-helo-%{version}.tar.bz2
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
BuildRequires:  gcc >= 3.4.6, make, bglibs-devel >= 1.104, mailfront-devel >= 1.12
BuildRequires:  libowfat-devel >= 0.29
Requires:       bglibs >= 1.104, mailfront >= 1.12, ucspi-tcp >= 0.88
Requires:       libowfat-shared >= 0.29
URL:            http://choon.net/mailfront-plugin-check-helo.php

%description
Plugin: check-helo

This plugin provides an ability to reject, defer or add it's HELO name
checking results to message header.

Reference:
http://www.linuxmagic.com/best_practices/valid_helo_domain.html

IMPORTANT: This plugin is written for and tested with mailfront 2.00
(might work with earlier version).

This plugin checks sequence:

(1).  If $HELO_CHECK_SKIP is set, this plugin will be skipped.

(2).  If $RELAYCLIENT is set, add to message header and skip the remaining
      checks.  Example of message header added:

      X-HELO-Check-Result: SKIPPED: $RELAYCLIENT is set for 1.2.3.4

(3).  If sender is authenticated, add to message header and skip the
      remaining checks.  Example of message header added:

      X-HELO-Check-Result: SKIPPED: authenticated user from 1.2.3.4

(4).  If $RBLSMTPD is set and empty, add to message header and skip the
      remaining checks.

(5).  If $HELO_CHECK_REMOTEHOST or $HELO_CHECK_REMOTEHOST_MATCH is set,
      this plugin will check ${PROTO}REMOTEHOST is not empty.  If is empty,
      will do a DNS PTR lookup using ${PROTO}REMOTEIP and use the return
      value as ${PROTO}REMOTEHOST.

(6).  If $HELO_CHECK_REMOTEHOST is set, this plugin will check HELO name
      is not empty.  If it is empty, add to message header or defer or
      reject accordingly and skip the remaining checks.
      (Refer to $HELO_CHECK_ADD_HEADER, $HELO_CHECK_DEFER and
      $HELO_CHECK_REJECT)
      Example of message header added:

      X-HELO-Check-Result: FAILED
      X-HELO-Check-Summary: REMOTEHOST is empty and PTR DNS lookup for IP 1.2.3.4 failed

(7).  If $HELO_CHECK_REMOTEHOST_MATCH is set, this plugin will check HELO
      name and ${PROTO}REMOTEHOST whether both are the same otherwise add
      to message header or defer or reject accordingly and skip the
      remaining checks.
      (Refer to $HELO_CHECK_ADD_HEADER, $HELO_CHECK_DEFER and
      $HELO_CHECK_REJECT)
      Example of message header added:

      X-HELO-Check-Result: FAILED
      X-HELO-Check-Summary: HELO name and REMOTEHOST doesn't match

(8).  Check HELO name for the following:
      (8.1).  Cannot starts with ".", "[" and "-"
      (8.2).  Cannot ends with ".", "]" and "-"
      (8.3).  Must have at least a "." (dot)
      (8.4).  Cannot have any of the following characters besides TAB
              and SPACE:
                  ~!@#$%^&*()_+={}[]:;"'<>,?/|
      (8.5).  Cannot use a valid IPv4 or IPv6 address
      NOTE: If any of the above checks failed, the remaining checks will
      be skipped.
      (Refer to $HELO_CHECK_ADD_HEADER, $HELO_CHECK_DEFER and
      $HELO_CHECK_REJECT)
      Example of message header added:

      X-HELO-Check-Result: FAILED
      X-HELO-Check-Summary: '.' found in last character of HELO name sender_set_HELO_name

(9).  Do a DNS lookup for HELO name for A, AAAA and PTR records.  If the
      DNS lookup failed due to no such name exists will skip all remaining
      checks and add to message header or defer or reject.
      (Refer to $HELO_CHECK_ADD_HEADER, $HELO_CHECK_DEFER and
      $HELO_CHECK_REJECT)
      Any other errors such as timeout etc. will result in skipping
      the remaining checks and message will be deferred with reply code
      451 if $HELO_CHECK_REJECT or $HELO_CHECK_DEFER is set otherwise
      if $HELO_CHECK_ADD_HEADER is set will add to message header.
      Example of message header added:

      X-HELO-Check-Result: FAILED
      X-HELO-Check-Summary: DNS lookup error for HELO name sender_set_HELO_name

(10). If $HELO_CHECK_REMOTEIP is set, this plugin will check for a match
      of ${PROTO}REMOTEIP and HELO name IP.  If there is a match, skip
      the remaining checks and add header if $HELO_CHECK_ADD_HEADER is set.
      If there is no match, add to message header or defer or reject.
      (Refer to $HELO_CHECK_ADD_HEADER, $HELO_CHECK_DEFER and
      $HELO_CHECK_REJECT)
      Example of message header added:

      X-HELO-Check-Result: PASSED
      X-HELO-Check-Summary: REMOTEIP matches sender_set_HELO_name's IP

(11). If $HELO_CHECK_REMOTEIP_MATCH is set, this plugin will check for a
      match within same class C for IPv4 and /64 of ${PROTO}REMOTEIP.
      If there isn't a match, add headers or defer or reject accordingly.
      If there is a match, add headers if $HELO_CHECK_ADD_HEADER and skip
      the rest of the checks.
      Example of message header added:

      X-HELO-Check-Result: SKIPPED
      X-HELO-Check-Summary: REMOTEIP 1.2.3.4 matches sender_set_HELO_name's IP 1.2.3.50 within same IP class

(12). If $HELO_CHECK_PTR is set, this plugin will check the HELO name's
      A records to have at least one PTR record. If the check failed,
      it will add to message header or defer or reject accordingly.
      Example of message header added:

      X-HELO-Check-Result: FAILED
      X-HELO-Check-Summary: X-HELO-Check-Summary: PTR record not found for HELO name sender_set_HELO_name

(13). If $HELO_CHECK_PTR_MATCH is set, this plugin will check there is at
      least one of the PTR record resolves to the same given HELO name.
      If the check failed, it will add to message header or defer or reject accordingly.
      Example of message header added:

      X-HELO-Check-Result: FAILED
      X-HELO-Check-Summary: X-HELO-Check-Summary: PTR record doesn't resolves back to HELO name sender_set_HELO_name

(14). If $HELO_CHECK_REJECT is set, sender will be rejected with reply
      code 553 if any of the above checks failed.

(15). If $HELO_CHECK_DEFER is set, sender will be deferred with reply
      code 451 if any of the above checks failed.
      NOTE: $HELO_CHECK_DEFER takes precedence over $HELO_CHECK_DEFER.

(16). If $HELO_CHECK_ADD_HEADER is set, this plugin will add its checking
      result to each mail message header accepted.

(17). If $HELO_CHECK_VERBOSE is set, this plugin will log all its checking
      to stderr.


%prep
%setup -q

make rpmenv
echo %{_libdir}/mailfront > conf-modules
echo %{_libdir}/bglibs > conf-bglibs
echo %{_includedir}/bglibs > conf-bgincs

%build
make

%install
rm -fr %{buildroot}
mkdir -p %{buildroot}/%{_libdir}/mailfront
install -m0755 plugin-check-helo.so %{buildroot}/%{_libdir}/mailfront/
#make install_prefix=%{buildroot} install

%clean
rm -rf %{buildroot}

%files
%defattr(-,root,root)
%doc README COPYING CHANGES plugin-check-helo.html
#%{_bindir}/*
%{_libdir}/mailfront/plugin-check-helo.so

%changelog

* Thu Aug 08 2013 Giam Teck Choon <choonrpms@choon.net> - 0.07-1
- Add IPv6 support

* Sun Jun 23 2013 Giam Teck Choon <choonrpms@choon.net> - 0.06-1
- Add new define elogerr function and simplified those printing to stderr log
- Change coding style to 2 spacing instead of using TAB as there is no reason
  to follow kernel coding style
- Add $HELO_CHECK_PTR_MATCH support
- Add $HELO_CHECK_ADD_HEADER support
- Update README and plugin-check-helo.html

* Fri Jun 07 2013 Giam Teck Choon <choonrpms@choon.net> - 0.05-1
- Change response numbers for defer (451) and reject (553) to follow what we
  used for rblsmtpd instead
- Add support to skip this plugin if $RBLSMTPD is set and empty
- Various bugs fixed mainly due to str_free() those not suppose to for static
  str declarations
- Rename declaration of str errormsg to str estr

* Fri May 31 2013 Giam Teck Choon <choonrpms@choon.net> - 0.04-1
- Fix typo in resp.number to 421 instead of 451 if $HELO_CHECK_DEFER is set
- Change return value in check_hostname function to actual ret value instead
  of 0
- If ret == EAI_MEMORY return Out of Memory error in response in function
  response *sender besides int outofmemory greater than 0
- If resp.number is greater than 0 and ret == EAI_NONAME, defer or reject
  accordingly in function response *sender otherwise defer the mail for
  remote MTA to retry later as failsafe

* Thu May 30 2013 Giam Teck Choon <choonrpms@choon.net> - 0.03-1
- Update plugin-check-helo.html
- Update README
- Add match_ipv4 function, implement $HELO_CHECK_REMOTEIP and
  $HELO_CHECK_REMOTEIP_MATCH
- Add init_HostDNSIP function, redo struct HostDNSIP and free_HostDNSIP
  function
- Remove a space
- Implement support environment variable $HELO_CHECK_VERBOSE

* Sat May 25 2013 Giam Teck Choon <choonrpms@choon.net> - 0.02-1
- Add define macro MAX_DNSIP_RECORDS default to 10 if not define and update
  function check_hostname to have this limit

* Fri May 24 2013 Giam Teck Choon <choonrpms@choon.net> - 0.01-1
- Initial Release
